@@ -34,11 +34,11 @@ Hypothesis::API - Wrapper for the hypothes.is web (HTTP) API.
 
 =head1 VERSION
 
-Version 0.09
+Version 0.10
 
 =cut
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 SYNOPSIS
 
@@ -208,21 +208,7 @@ sub create {
         $payload_out->{'tags'} = undef;
     }
     if (not exists $payload->{'target'}) {
-        $payload_out->{'target'} = {
-            "selector" => [
-                {
-                "start" => undef,
-                "end"   => undef,
-                "type"  => "TextPositionSelector"
-                },  
-                {
-                "type"   => "TextQuoteSelector", 
-                "prefix" => undef,
-                "exact"  => undef,
-                "suffix" => undef
-                },
-            ]
-        };
+        $payload_out->{'target'} = undef;
     }
     
     my $data = $json->encode($payload_out);
@@ -410,8 +396,17 @@ and $query->offset, which specifies the number of annotations
 to fetch at a time, but does not override the spirit of either
 of the $query parameters.
 
+Tries not to return annotations created after initiation
+of the search.
+
+Note that while this function has been made robust to addition of
+new annotations being created during a query, it is not yet
+robust to deletion of annotations.
+
 =cut
 
+# FIXME: improve handling of deletions
+
 sub search {
     my ($self, $query, $page_size) = @_;
 
@@ -441,7 +436,7 @@ sub search {
     my $next_buf_start;
     my $num_returned = 0;
     my $limit_orig = $query->{ 'limit' };
-    $query->{ 'limit' }  =  $page_size + 1;
+    $query->{ 'limit' } = $page_size + 1;
 
     my @annotation_buff = ();
     return sub {
@@ -457,22 +452,33 @@ sub search {
             $response = $self->ua->get( $url );
             $json_content = $json->decode($response->content);
             @annotation_buff = @{$json_content->{ 'rows' }};
-            if (@annotation_buff > $page_size) {
-                if (defined $next_buf_start) {
-                    while ($next_buf_start->{'id'} ne $annotation_buff[0]->{'id'}) {
-                        warn "mismatch: scanning for last seen id\n" if $VERB > 0;
-                        shift @annotation_buff;
-                        if (@annotation_buff == 0) {
-                            $query->{ 'offset' } += $page_size;
-                            goto QUERY;
-                        }
+            if ($limit_orig eq 'Infinity') {
+                # OK, we get the point, but let's get finite.
+                $limit_orig = $json_content->{ 'total' };
+                $query->{ 'limit' } = $json_content->{ 'total' };
+            }
+            if (defined $next_buf_start) {
+                # This assumes that the feed is like a stack: LIFO.
+                # Annotations created after the search call
+                # shouldn't be returned.
+                while (@annotation_buff && $next_buf_start->{'id'} ne $annotation_buff[0]->{'id'}) {
+                    warn "mismatch: scanning for last seen id\n" if $VERB > 0;
+                    shift @annotation_buff;
+                    if (@annotation_buff == 0) {
+                        $query->{ 'offset' } += $page_size;
+                        goto QUERY;
                     }
                 }
-                $next_buf_start = pop @annotation_buff;
             }
+            $next_buf_start = pop @annotation_buff;
             $done = 1 if (@annotation_buff == 0);
             $query->{ 'offset' } += $page_size;
             warn $response->content if $VERB > 5;
+            # Handle edge case that look-ahead element is the last element:
+            if (($num_returned + 1) == $limit_orig) {
+                $num_returned++;
+                return $next_buf_start;
+            }
         }
         $num_returned++;
         return undef if $done;
